package org.papervision3d.materials
{
   import flash.display.BitmapData;
   import flash.display.Graphics;
   import flash.geom.Matrix;
   import flash.geom.Point;
   import flash.geom.Rectangle;
   import flash.utils.Dictionary;
   import org.papervision3d.Papervision3D;
   import org.papervision3d.core.draw.IFaceDrawer;
   import org.papervision3d.core.geom.Face3D;
   import org.papervision3d.core.geom.Vertex2D;
   import org.papervision3d.core.proto.MaterialObject3D;
   import org.papervision3d.objects.DisplayObject3D;
   
   public class BitmapMaterial extends MaterialObject3D implements IFaceDrawer
   {
      public static var AUTO_MIP_MAPPING:Boolean = true;
      
      public static var MIP_MAP_DEPTH:Number = 8;
      
      protected static var _triMatrix:Matrix = new Matrix();
      
      protected static var _localMatrix:Matrix = new Matrix();
      
      public var uvMatrices:Dictionary;
      
      protected var _texture:*;
      
      public function BitmapMaterial(param1:*, param2:Object = null)
      {
         super(param2);
         texture = param1;
         init();
      }
      
      private function init() : void
      {
         uvMatrices = new Dictionary();
      }
      
      public function transformUV(param1:Face3D, param2:DisplayObject3D = null) : Matrix
      {
         var _loc3_:Array = null;
         var _loc4_:Number = NaN;
         var _loc5_:Number = NaN;
         var _loc6_:Number = NaN;
         var _loc7_:Number = NaN;
         var _loc8_:Number = NaN;
         var _loc9_:Number = NaN;
         var _loc10_:Number = NaN;
         var _loc11_:Number = NaN;
         var _loc12_:Number = NaN;
         var _loc13_:Number = NaN;
         var _loc14_:Number = NaN;
         var _loc15_:Number = NaN;
         var _loc16_:Matrix = null;
         var _loc17_:Matrix = null;
         if(!param1.uv)
         {
            Papervision3D.log("MaterialObject3D: transformUV() uv not found!");
         }
         else if(bitmap)
         {
            _loc3_ = param1.uv;
            _loc4_ = bitmap.width * maxU;
            _loc5_ = bitmap.height * maxV;
            _loc6_ = _loc4_ * _loc3_[0].u;
            _loc7_ = _loc5_ * (1 - _loc3_[0].v);
            _loc8_ = _loc4_ * _loc3_[1].u;
            _loc9_ = _loc5_ * (1 - _loc3_[1].v);
            _loc10_ = _loc4_ * _loc3_[2].u;
            _loc11_ = _loc5_ * (1 - _loc3_[2].v);
            if(_loc6_ == _loc8_ && _loc7_ == _loc9_ || _loc6_ == _loc10_ && _loc7_ == _loc11_)
            {
               _loc6_ -= _loc6_ > 0.05 ? 0.05 : -0.05;
               _loc7_ -= _loc7_ > 0.07 ? 0.07 : -0.07;
            }
            if(_loc10_ == _loc8_ && _loc11_ == _loc9_)
            {
               _loc10_ -= _loc10_ > 0.05 ? 0.04 : -0.04;
               _loc11_ -= _loc11_ > 0.06 ? 0.06 : -0.06;
            }
            _loc12_ = _loc8_ - _loc6_;
            _loc13_ = _loc9_ - _loc7_;
            _loc14_ = _loc10_ - _loc6_;
            _loc15_ = _loc11_ - _loc7_;
            _loc16_ = new Matrix(_loc12_,_loc13_,_loc14_,_loc15_,_loc6_,_loc7_);
            _loc16_.invert();
            _loc17_ = uvMatrices[param1] || (uvMatrices[param1] = _loc16_.clone());
            _loc17_.a = _loc16_.a;
            _loc17_.b = _loc16_.b;
            _loc17_.c = _loc16_.c;
            _loc17_.d = _loc16_.d;
            _loc17_.tx = _loc16_.tx;
            _loc17_.ty = _loc16_.ty;
         }
         else
         {
            Papervision3D.log("MaterialObject3D: transformUV() material.bitmap not found!");
         }
         return _loc17_;
      }
      
      public function get texture() : *
      {
         return this._texture;
      }
      
      public function correctBitmap(param1:BitmapData, param2:Boolean) : BitmapData
      {
         var _loc3_:BitmapData = null;
         var _loc4_:Number = NaN;
         var _loc5_:Number = NaN;
         var _loc6_:Number = NaN;
         var _loc7_:Boolean = false;
         _loc4_ = 1 << MIP_MAP_DEPTH;
         _loc5_ = _loc4_ * Math.ceil(param1.width / _loc4_);
         _loc6_ = _loc4_ * Math.ceil(param1.height / _loc4_);
         _loc7_ = true;
         if(_loc5_ > 2880)
         {
            _loc5_ = param1.width;
            _loc7_ = false;
         }
         if(_loc6_ > 2880)
         {
            _loc6_ = param1.height;
            _loc7_ = false;
         }
         if(!_loc7_)
         {
            Papervision3D.log("Material " + this.name + ": Texture too big for mip mapping. Resizing recommended for better performance and quality.");
         }
         if(AUTO_MIP_MAPPING && param1 && (param1.width % _loc4_ != 0 || param1.height % _loc4_ != 0))
         {
            this.maxU = param1.width / _loc5_;
            this.maxV = param1.height / _loc6_;
            _loc3_ = new BitmapData(_loc5_,_loc6_,param1.transparent,0);
            _loc3_.draw(param1);
            extendBitmapEdges(_loc3_,param1.width,param1.height);
            if(param2)
            {
               param1.dispose();
            }
         }
         else
         {
            this.maxU = this.maxV = 1;
            _loc3_ = param1;
         }
         return _loc3_;
      }
      
      public function set texture(param1:*) : void
      {
         this.bitmap = createBitmap(param1);
         this._texture = param1;
      }
      
      protected function createBitmap(param1:*) : BitmapData
      {
         return correctBitmap(param1,false);
      }
      
      override public function toString() : String
      {
         return "Texture:" + this.texture + " lineColor:" + this.lineColor + " lineAlpha:" + this.lineAlpha;
      }
      
      override public function drawFace3D(param1:DisplayObject3D, param2:Face3D, param3:Graphics, param4:Vertex2D, param5:Vertex2D, param6:Vertex2D) : int
      {
         var _loc7_:Matrix = null;
         var _loc8_:Number = NaN;
         var _loc9_:Number = NaN;
         var _loc10_:Number = NaN;
         var _loc11_:Number = NaN;
         var _loc12_:Number = NaN;
         var _loc13_:Number = NaN;
         if(bitmap)
         {
            _loc7_ = uvMatrices[param2] || transformUV(param2,param1);
            _loc8_ = param4.x;
            _loc9_ = param4.y;
            _loc10_ = param5.x;
            _loc11_ = param5.y;
            _loc12_ = param6.x;
            _loc13_ = param6.y;
            _triMatrix.a = _loc10_ - _loc8_;
            _triMatrix.b = _loc11_ - _loc9_;
            _triMatrix.c = _loc12_ - _loc8_;
            _triMatrix.d = _loc13_ - _loc9_;
            _triMatrix.tx = _loc8_;
            _triMatrix.ty = _loc9_;
            _localMatrix.a = _loc7_.a;
            _localMatrix.b = _loc7_.b;
            _localMatrix.c = _loc7_.c;
            _localMatrix.d = _loc7_.d;
            _localMatrix.tx = _loc7_.tx;
            _localMatrix.ty = _loc7_.ty;
            _localMatrix.concat(_triMatrix);
            param3.beginBitmapFill(bitmap,_localMatrix,true,smooth);
            param3.moveTo(_loc8_,_loc9_);
            param3.lineTo(_loc10_,_loc11_);
            param3.lineTo(_loc12_,_loc13_);
            param3.lineTo(_loc8_,_loc9_);
            param3.endFill();
         }
         return 1;
      }
      
      override public function copy(param1:MaterialObject3D) : void
      {
         super.copy(param1);
         this.maxU = param1.maxU;
         this.maxV = param1.maxV;
      }
      
      protected function extendBitmapEdges(param1:BitmapData, param2:Number, param3:Number) : void
      {
         var _loc4_:Rectangle = null;
         var _loc5_:Point = null;
         var _loc6_:int = 0;
         _loc4_ = new Rectangle();
         _loc5_ = new Point();
         if(param1.width > param2)
         {
            _loc4_.x = param2 - 1;
            _loc4_.y = 0;
            _loc4_.width = 1;
            _loc4_.height = param3;
            _loc5_.y = 0;
            _loc6_ = param2;
            while(_loc6_ < param1.width)
            {
               _loc5_.x = _loc6_;
               param1.copyPixels(param1,_loc4_,_loc5_);
               _loc6_++;
            }
         }
         if(param1.height > param3)
         {
            _loc4_.x = 0;
            _loc4_.y = param3 - 1;
            _loc4_.width = param1.width;
            _loc4_.height = 1;
            _loc5_.x = 0;
            _loc6_ = param3;
            while(_loc6_ < param1.height)
            {
               _loc5_.y = _loc6_;
               param1.copyPixels(param1,_loc4_,_loc5_);
               _loc6_++;
            }
         }
      }
      
      override public function clone() : MaterialObject3D
      {
         var _loc1_:MaterialObject3D = null;
         _loc1_ = super.clone();
         _loc1_.maxU = this.maxU;
         _loc1_.maxV = this.maxV;
         return _loc1_;
      }
   }
}

