package org.papervision3d.objects
{
   import com.blitzagency.xray.logger.XrayLog;
   import flash.display.Sprite;
   import org.papervision3d.Papervision3D;
   import org.papervision3d.core.Matrix3D;
   import org.papervision3d.core.Number3D;
   import org.papervision3d.core.geom.Face3DInstance;
   import org.papervision3d.core.proto.CameraObject3D;
   import org.papervision3d.core.proto.DisplayObjectContainer3D;
   import org.papervision3d.core.proto.GeometryObject3D;
   import org.papervision3d.core.proto.MaterialObject3D;
   import org.papervision3d.core.proto.SceneObject3D;
   import org.papervision3d.materials.MaterialsList;
   import org.papervision3d.scenes.InteractiveScene3D;
   import org.papervision3d.utils.InteractiveSceneManager;
   
   public class DisplayObject3D extends DisplayObjectContainer3D
   {
      public static const MESH_SORT_CENTER:uint = 1;
      
      public static const MESH_SORT_FAR:uint = 2;
      
      public static const MESH_SORT_CLOSE:uint = 3;
      
      private static var FORWARD:Number3D = new Number3D(0,0,1);
      
      private static var BACKWARD:Number3D = new Number3D(0,0,-1);
      
      private static var LEFT:Number3D = new Number3D(-1,0,0);
      
      private static var RIGHT:Number3D = new Number3D(1,0,0);
      
      private static var UP:Number3D = new Number3D(0,1,0);
      
      private static var DOWN:Number3D = new Number3D(0,-1,0);
      
      private static var _totalDisplayObjects:int = 0;
      
      private static var toDEGREES:Number = 180 / Math.PI;
      
      private static var toRADIANS:Number = Math.PI / 180;
      
      public var extra:Object;
      
      public var id:int;
      
      private var _rotationY:Number;
      
      private var _rotationZ:Number;
      
      private var _rotationX:Number;
      
      public var material:MaterialObject3D;
      
      public var meshSort:uint = 1;
      
      public var materials:MaterialsList;
      
      private var _scaleDirty:Boolean = false;
      
      private var _scaleX:Number;
      
      public var screenZ:Number;
      
      public var geometry:GeometryObject3D;
      
      private var _scaleY:Number;
      
      private var _scaleZ:Number;
      
      public var visible:Boolean;
      
      public var transform:Matrix3D;
      
      public var interactiveSceneManager:InteractiveSceneManager;
      
      public var name:String;
      
      public var container:Sprite;
      
      public var parent:DisplayObjectContainer3D;
      
      public var view:Matrix3D;
      
      protected var _scene:SceneObject3D = null;
      
      public var faces:Array;
      
      protected var _transformDirty:Boolean = false;
      
      private var _rotationDirty:Boolean = false;
      
      protected var _sorted:Array;
      
      public function DisplayObject3D(param1:String = null, param2:GeometryObject3D = null, param3:Object = null)
      {
         var _loc4_:Number = NaN;
         _scene = null;
         meshSort = MESH_SORT_CENTER;
         faces = new Array();
         _transformDirty = false;
         _rotationDirty = false;
         _scaleDirty = false;
         super();
         Papervision3D.log("DisplayObject3D: " + param1);
         this.transform = Matrix3D.IDENTITY;
         this.view = Matrix3D.IDENTITY;
         this.x = !!param3 ? Number(param3.x) || 0 : 0;
         this.y = !!param3 ? Number(param3.y) || 0 : 0;
         this.z = !!param3 ? Number(param3.z) || 0 : 0;
         rotationX = !!param3 ? Number(param3.rotationX) || 0 : 0;
         rotationY = !!param3 ? Number(param3.rotationY) || 0 : 0;
         rotationZ = !!param3 ? Number(param3.rotationZ) || 0 : 0;
         _loc4_ = Papervision3D.usePERCENT ? 100 : 1;
         scaleX = !!param3 ? Number(param3.scaleX) || _loc4_ : _loc4_;
         scaleY = !!param3 ? Number(param3.scaleY) || _loc4_ : _loc4_;
         scaleZ = !!param3 ? Number(param3.scaleZ) || _loc4_ : _loc4_;
         if(Boolean(param3) && Boolean(param3.extra))
         {
            this.extra = param3.extra;
         }
         if(Boolean(param3) && Boolean(param3.container))
         {
            this.container = param3.container;
         }
         this.visible = true;
         this.id = _totalDisplayObjects++;
         this.name = param1 || String(this.id);
         if(param2)
         {
            addGeometry(param2);
         }
      }
      
      public static function get ZERO() : DisplayObject3D
      {
         return new DisplayObject3D();
      }
      
      public function set z(param1:Number) : void
      {
         this.transform.n34 = param1;
      }
      
      override public function addChild(param1:DisplayObject3D, param2:String = null) : DisplayObject3D
      {
         param1 = super.addChild(param1,param2);
         if(param1.scene == null)
         {
            param1.scene = scene;
         }
         return param1;
      }
      
      public function moveDown(param1:Number) : void
      {
         translate(param1,DOWN);
      }
      
      public function set scene(param1:SceneObject3D) : void
      {
         var _loc2_:DisplayObject3D = null;
         _scene = param1;
         for each(_loc2_ in this._childrenByName)
         {
            if(_loc2_.scene == null)
            {
               _loc2_.scene = _scene;
            }
         }
         if(_scene is InteractiveScene3D == false)
         {
            return;
         }
         interactiveSceneManager = InteractiveScene3D(_scene).interactiveSceneManager;
      }
      
      public function project(param1:DisplayObject3D, param2:CameraObject3D, param3:Array = null) : Number
      {
         var _loc4_:Number = NaN;
         var _loc5_:Number = NaN;
         var _loc6_:DisplayObject3D = null;
         if(!param3)
         {
            this._sorted = param3 = new Array();
         }
         if(this._transformDirty)
         {
            updateTransform();
         }
         this.view.calculateMultiply(param1.view,this.transform);
         _loc4_ = 0;
         _loc5_ = 0;
         for each(_loc6_ in this._childrenByName)
         {
            if(_loc6_.visible)
            {
               _loc4_ += _loc6_.project(this,param2,param3);
               _loc5_++;
            }
         }
         return this.screenZ = _loc4_ / _loc5_;
      }
      
      public function lookAt(param1:DisplayObject3D, param2:Number3D = null) : void
      {
         var _loc3_:Number3D = null;
         var _loc4_:Number3D = null;
         var _loc5_:Number3D = null;
         var _loc6_:Number3D = null;
         var _loc7_:Number3D = null;
         var _loc8_:Matrix3D = null;
         var _loc9_:XrayLog = null;
         _loc3_ = new Number3D(this.x,this.y,this.z);
         _loc4_ = new Number3D(param1.x,param1.y,param1.z);
         _loc5_ = Number3D.sub(_loc4_,_loc3_);
         _loc5_.normalize();
         if(_loc5_.modulo > 0.1)
         {
            _loc6_ = Number3D.cross(_loc5_,param2 || UP);
            _loc6_.normalize();
            _loc7_ = Number3D.cross(_loc5_,_loc6_);
            _loc7_.normalize();
            _loc8_ = this.transform;
            _loc8_.n11 = _loc6_.x;
            _loc8_.n21 = _loc6_.y;
            _loc8_.n31 = _loc6_.z;
            _loc8_.n12 = -_loc7_.x;
            _loc8_.n22 = -_loc7_.y;
            _loc8_.n32 = -_loc7_.z;
            _loc8_.n13 = _loc5_.x;
            _loc8_.n23 = _loc5_.y;
            _loc8_.n33 = _loc5_.z;
            this._transformDirty = false;
            this._rotationDirty = true;
         }
         else
         {
            _loc9_ = new XrayLog();
            _loc9_.debug("lookAt Error");
         }
      }
      
      public function set rotationX(param1:Number) : void
      {
         this._rotationX = Papervision3D.useDEGREES ? -param1 * toRADIANS : -param1;
         this._transformDirty = true;
      }
      
      public function set rotationY(param1:Number) : void
      {
         this._rotationY = Papervision3D.useDEGREES ? -param1 * toRADIANS : -param1;
         this._transformDirty = true;
      }
      
      public function set rotationZ(param1:Number) : void
      {
         this._rotationZ = Papervision3D.useDEGREES ? -param1 * toRADIANS : -param1;
         this._transformDirty = true;
      }
      
      public function addGeometry(param1:GeometryObject3D = null) : void
      {
         if(param1)
         {
            this.geometry = param1;
         }
      }
      
      public function moveUp(param1:Number) : void
      {
         translate(param1,UP);
      }
      
      public function get scaleX() : Number
      {
         if(Papervision3D.usePERCENT)
         {
            return this._scaleX * 100;
         }
         return this._scaleX;
      }
      
      public function get scaleY() : Number
      {
         if(Papervision3D.usePERCENT)
         {
            return this._scaleY * 100;
         }
         return this._scaleY;
      }
      
      public function get scaleZ() : Number
      {
         if(Papervision3D.usePERCENT)
         {
            return this._scaleZ * 100;
         }
         return this._scaleZ;
      }
      
      public function distanceTo(param1:DisplayObject3D) : Number
      {
         var _loc2_:Number = NaN;
         var _loc3_:Number = NaN;
         var _loc4_:Number = NaN;
         _loc2_ = this.x - param1.x;
         _loc3_ = this.y - param1.y;
         _loc4_ = this.z - param1.z;
         return Math.sqrt(_loc2_ * _loc2_ + _loc3_ * _loc3_ + _loc4_ * _loc4_);
      }
      
      public function get scale() : Number
      {
         if(this._scaleX == this._scaleY && this._scaleX == this._scaleZ)
         {
            if(Papervision3D.usePERCENT)
            {
               return this._scaleX * 100;
            }
            return this._scaleX;
         }
         return NaN;
      }
      
      public function translate(param1:Number, param2:Number3D) : void
      {
         var _loc3_:Number3D = null;
         _loc3_ = param2.clone();
         if(this._transformDirty)
         {
            updateTransform();
         }
         Matrix3D.rotateAxis(transform,_loc3_);
         this.x += param1 * _loc3_.x;
         this.y += param1 * _loc3_.y;
         this.z += param1 * _loc3_.z;
      }
      
      public function pitch(param1:Number) : void
      {
         var _loc2_:Number3D = null;
         var _loc3_:Matrix3D = null;
         param1 = Papervision3D.useDEGREES ? param1 * toRADIANS : param1;
         _loc2_ = RIGHT.clone();
         if(this._transformDirty)
         {
            updateTransform();
         }
         Matrix3D.rotateAxis(transform,_loc2_);
         _loc3_ = Matrix3D.rotationMatrix(_loc2_.x,_loc2_.y,_loc2_.z,param1);
         this.transform.calculateMultiply3x3(_loc3_,transform);
         this._rotationDirty = true;
      }
      
      public function hitTestObject(param1:DisplayObject3D, param2:Number = 1) : Boolean
      {
         var _loc3_:Number = NaN;
         var _loc4_:Number = NaN;
         var _loc5_:Number = NaN;
         var _loc6_:Number = NaN;
         var _loc7_:Number = NaN;
         var _loc8_:Number = NaN;
         _loc3_ = this.x - param1.x;
         _loc4_ = this.y - param1.y;
         _loc5_ = this.z - param1.z;
         _loc6_ = _loc3_ * _loc3_ + _loc4_ * _loc4_ + _loc5_ * _loc5_;
         _loc7_ = !!this.geometry ? this.geometry.boundingSphere2 : 0;
         _loc8_ = !!param1.geometry ? param1.geometry.boundingSphere2 : 0;
         _loc7_ *= param2;
         return _loc7_ + _loc8_ > _loc6_;
      }
      
      private function updateRotation() : void
      {
         var _loc1_:Number3D = null;
         _loc1_ = Matrix3D.matrix2euler(this.transform);
         this._rotationX = _loc1_.x * toRADIANS;
         this._rotationY = _loc1_.y * toRADIANS;
         this._rotationZ = _loc1_.z * toRADIANS;
         this._rotationDirty = false;
      }
      
      public function yaw(param1:Number) : void
      {
         var _loc2_:Number3D = null;
         var _loc3_:Matrix3D = null;
         param1 = Papervision3D.useDEGREES ? param1 * toRADIANS : param1;
         _loc2_ = UP.clone();
         if(this._transformDirty)
         {
            updateTransform();
         }
         Matrix3D.rotateAxis(transform,_loc2_);
         _loc3_ = Matrix3D.rotationMatrix(_loc2_.x,_loc2_.y,_loc2_.z,param1);
         this.transform.calculateMultiply3x3(_loc3_,transform);
         this._rotationDirty = true;
      }
      
      override public function toString() : String
      {
         return this.name + ": x:" + Math.round(this.x) + " y:" + Math.round(this.y) + " z:" + Math.round(this.z);
      }
      
      public function moveLeft(param1:Number) : void
      {
         translate(param1,LEFT);
      }
      
      public function get z() : Number
      {
         return this.transform.n34;
      }
      
      public function roll(param1:Number) : void
      {
         var _loc2_:Number3D = null;
         var _loc3_:Matrix3D = null;
         param1 = Papervision3D.useDEGREES ? param1 * toRADIANS : param1;
         _loc2_ = FORWARD.clone();
         if(this._transformDirty)
         {
            updateTransform();
         }
         Matrix3D.rotateAxis(transform,_loc2_);
         _loc3_ = Matrix3D.rotationMatrix(_loc2_.x,_loc2_.y,_loc2_.z,param1);
         this.transform.calculateMultiply3x3(_loc3_,transform);
         this._rotationDirty = true;
      }
      
      public function copyTransform(param1:*) : void
      {
         var _loc2_:Matrix3D = null;
         var _loc3_:Matrix3D = null;
         _loc2_ = this.transform;
         _loc3_ = param1 is DisplayObject3D ? param1.transform : param1;
         _loc2_.n11 = _loc3_.n11;
         _loc2_.n12 = _loc3_.n12;
         _loc2_.n13 = _loc3_.n13;
         _loc2_.n14 = _loc3_.n14;
         _loc2_.n21 = _loc3_.n21;
         _loc2_.n22 = _loc3_.n22;
         _loc2_.n23 = _loc3_.n23;
         _loc2_.n24 = _loc3_.n24;
         _loc2_.n31 = _loc3_.n31;
         _loc2_.n32 = _loc3_.n32;
         _loc2_.n33 = _loc3_.n33;
         _loc2_.n34 = _loc3_.n34;
         this._transformDirty = false;
         this._rotationDirty = true;
      }
      
      public function getMaterialByName(param1:String) : MaterialObject3D
      {
         var _loc2_:MaterialObject3D = null;
         var _loc3_:DisplayObject3D = null;
         _loc2_ = this.materials.getMaterialByName(param1);
         if(_loc2_)
         {
            return _loc2_;
         }
         for each(_loc3_ in this._childrenByName)
         {
            _loc2_ = _loc3_.getMaterialByName(param1);
            if(_loc2_)
            {
               return _loc2_;
            }
         }
         return null;
      }
      
      public function get x() : Number
      {
         return this.transform.n14;
      }
      
      public function get scene() : SceneObject3D
      {
         return _scene;
      }
      
      public function get y() : Number
      {
         return this.transform.n24;
      }
      
      public function set scale(param1:Number) : void
      {
         if(Papervision3D.usePERCENT)
         {
            param1 /= 100;
         }
         this._scaleX = this._scaleY = this._scaleZ = param1;
         this._transformDirty = true;
      }
      
      public function render(param1:SceneObject3D) : void
      {
         var _loc2_:Array = null;
         var _loc3_:Sprite = null;
         var _loc4_:Number = NaN;
         var _loc5_:Face3DInstance = null;
         var _loc6_:int = 0;
         _loc2_ = this._sorted;
         _loc2_.sortOn("screenZ",Array.DESCENDING | Array.NUMERIC);
         _loc3_ = this.container || param1.container;
         _loc4_ = 0;
         _loc6_ = 0;
         while(true)
         {
            _loc5_ = _loc2_[_loc6_];
            if(!_loc5_)
            {
               break;
            }
            if(_loc5_.visible)
            {
               _loc4_ += _loc5_.face.render(_loc5_.instance,_loc3_);
            }
            _loc6_++;
         }
         param1.stats.rendered += _loc4_;
      }
      
      public function get rotationY() : Number
      {
         if(this._rotationDirty)
         {
            updateRotation();
         }
         return Papervision3D.useDEGREES ? -this._rotationY * toDEGREES : -this._rotationY;
      }
      
      public function get rotationZ() : Number
      {
         if(this._rotationDirty)
         {
            updateRotation();
         }
         return Papervision3D.useDEGREES ? -this._rotationZ * toDEGREES : -this._rotationZ;
      }
      
      public function set scaleY(param1:Number) : void
      {
         if(Papervision3D.usePERCENT)
         {
            this._scaleY = param1 / 100;
         }
         else
         {
            this._scaleY = param1;
         }
         this._transformDirty = true;
      }
      
      public function set scaleZ(param1:Number) : void
      {
         if(Papervision3D.usePERCENT)
         {
            this._scaleZ = param1 / 100;
         }
         else
         {
            this._scaleZ = param1;
         }
         this._transformDirty = true;
      }
      
      public function get rotationX() : Number
      {
         if(this._rotationDirty)
         {
            updateRotation();
         }
         return Papervision3D.useDEGREES ? -this._rotationX * toDEGREES : -this._rotationX;
      }
      
      public function set scaleX(param1:Number) : void
      {
         if(Papervision3D.usePERCENT)
         {
            this._scaleX = param1 / 100;
         }
         else
         {
            this._scaleX = param1;
         }
         this._transformDirty = true;
      }
      
      protected function updateTransform() : void
      {
         var _loc1_:Object = null;
         var _loc2_:Matrix3D = null;
         var _loc3_:Matrix3D = null;
         var _loc4_:Matrix3D = null;
         _loc1_ = Matrix3D.euler2quaternion(-this._rotationY,-this._rotationZ,this._rotationX);
         _loc2_ = Matrix3D.quaternion2matrix(_loc1_.x,_loc1_.y,_loc1_.z,_loc1_.w);
         _loc3_ = this.transform;
         _loc2_.n14 = _loc3_.n14;
         _loc2_.n24 = _loc3_.n24;
         _loc2_.n34 = _loc3_.n34;
         _loc3_.copy(_loc2_);
         _loc4_ = Matrix3D.IDENTITY;
         _loc4_.n11 = this._scaleX;
         _loc4_.n22 = this._scaleY;
         _loc4_.n33 = this._scaleZ;
         this.transform.calculateMultiply(_loc3_,_loc4_);
         this._transformDirty = false;
      }
      
      public function moveForward(param1:Number) : void
      {
         translate(param1,FORWARD);
      }
      
      public function copyPosition(param1:*) : void
      {
         var _loc2_:Matrix3D = null;
         var _loc3_:Matrix3D = null;
         _loc2_ = this.transform;
         _loc3_ = param1 is DisplayObject3D ? param1.transform : param1;
         _loc2_.n14 = _loc3_.n14;
         _loc2_.n24 = _loc3_.n24;
         _loc2_.n34 = _loc3_.n34;
      }
      
      public function hitTestPoint(param1:Number, param2:Number, param3:Number) : Boolean
      {
         var _loc4_:Number = NaN;
         var _loc5_:Number = NaN;
         var _loc6_:Number = NaN;
         var _loc7_:Number = NaN;
         var _loc8_:Number = NaN;
         _loc4_ = this.x - param1;
         _loc5_ = this.y - param2;
         _loc6_ = this.z - param3;
         _loc7_ = param1 * param1 + param2 * param2 + param3 * param3;
         _loc8_ = !!this.geometry ? this.geometry.boundingSphere2 : 0;
         return _loc8_ > _loc7_;
      }
      
      public function moveRight(param1:Number) : void
      {
         translate(param1,RIGHT);
      }
      
      public function moveBackward(param1:Number) : void
      {
         translate(param1,BACKWARD);
      }
      
      public function materialsList() : String
      {
         var _loc1_:String = null;
         var _loc2_:String = null;
         var _loc3_:DisplayObject3D = null;
         _loc1_ = "";
         for(_loc2_ in this.materials)
         {
            _loc1_ += _loc2_ + "\n";
         }
         for each(_loc3_ in this._childrenByName)
         {
            for(_loc2_ in _loc3_.materials.materialsByName)
            {
               _loc1_ += "+ " + _loc2_ + "\n";
            }
         }
         return _loc1_;
      }
      
      public function set x(param1:Number) : void
      {
         this.transform.n14 = param1;
      }
      
      public function set y(param1:Number) : void
      {
         this.transform.n24 = param1;
      }
   }
}

