package org.papervision3d.utils
{
   import com.blitzagency.xray.logger.XrayLog;
   import flash.display.Sprite;
   import flash.events.Event;
   import flash.events.EventDispatcher;
   import flash.events.MouseEvent;
   import flash.utils.Dictionary;
   import org.papervision3d.core.proto.SceneObject3D;
   import org.papervision3d.events.InteractiveScene3DEvent;
   import org.papervision3d.objects.DisplayObject3D;
   
   public class InteractiveSceneManager extends EventDispatcher
   {
      public static var DEFAULT_SPRITE_ALPHA:Number = 0.0051;
      
      public static var DEFAULT_FILL_ALPHA:Number = 0.0051;
      
      public static var DEFAULT_FILL_COLOR:Number = 16777215;
      
      protected var log:XrayLog;
      
      public var container:Sprite;
      
      public var scene:SceneObject3D;
      
      public var faceDictionary:Dictionary;
      
      public var debug:Boolean = false;
      
      protected var dummy:DisplayObject3D;
      
      public var containerDictionary:Dictionary;
      
      public function InteractiveSceneManager(param1:SceneObject3D)
      {
         faceDictionary = new Dictionary();
         containerDictionary = new Dictionary();
         container = new Sprite();
         debug = false;
         dummy = DisplayObject3D.ZERO;
         log = new XrayLog();
         super();
         scene = param1;
         scene.container.parent.addChild(container);
         container.x = scene.container.x;
         container.y = scene.container.y;
         container.stage.addEventListener(Event.RESIZE,handleResize);
         container.stage.addEventListener(MouseEvent.MOUSE_UP,handleReleaseOutside);
      }
      
      protected function sortObjects() : void
      {
         var _loc1_:Array = null;
         var _loc2_:InteractiveContainerData = null;
         var _loc3_:Number = NaN;
         var _loc4_:Number = NaN;
         _loc1_ = [];
         for each(_loc2_ in faceDictionary)
         {
            if(_loc2_.sort)
            {
               _loc4_ = _loc2_.displayObject3D.screenZ;
               _loc1_.push({
                  "container":_loc2_.container,
                  "distance":_loc4_
               });
            }
         }
         _loc1_.sortOn("distance",Array.DESCENDING | Array.NUMERIC);
         _loc3_ = 0;
         while(_loc3_ < _loc1_.length)
         {
            container.addChild(_loc1_[_loc3_].container);
            _loc3_++;
         }
      }
      
      public function drawFace(param1:DisplayObject3D, param2:Number, param3:Number, param4:Number, param5:Number, param6:Number, param7:Number) : void
      {
         var _loc8_:InteractiveContainerData = null;
         if(faceDictionary[param1] == null)
         {
            addDisplayObject(param1);
         }
         _loc8_ = faceDictionary[param1];
         _loc8_.container.graphics.beginFill(_loc8_.color,_loc8_.fillAlpha);
         _loc8_.container.graphics.moveTo(param2,param5);
         _loc8_.container.graphics.lineTo(param3,param6);
         _loc8_.container.graphics.lineTo(param4,param7);
         _loc8_.container.graphics.endFill();
         _loc8_.isDrawn = true;
      }
      
      public function getDisplayObject3D(param1:Sprite) : DisplayObject3D
      {
         return DisplayObject3D(containerDictionary[param1]);
      }
      
      public function getSprite(param1:DisplayObject3D) : Sprite
      {
         return InteractiveContainerData(faceDictionary[param1]).container;
      }
      
      protected function handleMouseOver(param1:MouseEvent) : void
      {
         if(debug)
         {
            log.debug("Over",DisplayObject3D(containerDictionary[param1.target]).name);
         }
         dispatchEvent(new InteractiveScene3DEvent(InteractiveScene3DEvent.OBJECT_OVER,containerDictionary[param1.currentTarget],Sprite(param1.currentTarget)));
      }
      
      public function addDisplayObject(param1:DisplayObject3D) : void
      {
         var _loc2_:InteractiveContainerData = null;
         if(faceDictionary[param1] == null)
         {
            _loc2_ = faceDictionary[param1] = new InteractiveContainerData(param1);
            containerDictionary[_loc2_.container] = param1;
            _loc2_.container.addEventListener(MouseEvent.MOUSE_DOWN,handleMousePress);
            _loc2_.container.addEventListener(MouseEvent.MOUSE_UP,handleMouseRelease);
            _loc2_.container.addEventListener(MouseEvent.CLICK,handleMouseClick);
            _loc2_.container.addEventListener(MouseEvent.MOUSE_OVER,handleMouseOver);
            _loc2_.container.addEventListener(MouseEvent.MOUSE_OUT,handleMouseOut);
            _loc2_.container.addEventListener(MouseEvent.MOUSE_MOVE,handleMouseMove);
            if(debug)
            {
               log.debug("addDisplayObject id",param1.id,param1.name,DEFAULT_SPRITE_ALPHA);
            }
         }
      }
      
      public function resetFaces() : void
      {
         var _loc1_:InteractiveContainerData = null;
         for each(_loc1_ in faceDictionary)
         {
            _loc1_.container.graphics.clear();
            _loc1_.sort = _loc1_.isDrawn;
            _loc1_.isDrawn = false;
         }
         sortObjects();
      }
      
      public function resizeStage() : void
      {
         container.x = scene.container.x;
         container.y = scene.container.y;
      }
      
      protected function handleMouseClick(param1:MouseEvent) : void
      {
         if(debug)
         {
            log.debug("click",DisplayObject3D(containerDictionary[param1.target]).name);
         }
         dispatchEvent(new InteractiveScene3DEvent(InteractiveScene3DEvent.OBJECT_CLICK,containerDictionary[param1.currentTarget],Sprite(param1.currentTarget)));
      }
      
      protected function handleMouseMove(param1:MouseEvent) : void
      {
         if(debug)
         {
            log.debug("Move",DisplayObject3D(containerDictionary[param1.target]).name);
         }
         dispatchEvent(new InteractiveScene3DEvent(InteractiveScene3DEvent.OBJECT_MOVE,containerDictionary[param1.currentTarget],Sprite(param1.currentTarget)));
      }
      
      protected function handleMouseRelease(param1:MouseEvent) : void
      {
         if(debug)
         {
            log.debug("release",DisplayObject3D(containerDictionary[param1.target]).name);
         }
         dispatchEvent(new InteractiveScene3DEvent(InteractiveScene3DEvent.OBJECT_RELEASE,containerDictionary[param1.currentTarget],Sprite(param1.currentTarget)));
      }
      
      protected function handleMousePress(param1:MouseEvent) : void
      {
         if(debug)
         {
            log.debug("press",DisplayObject3D(containerDictionary[param1.target]).name);
         }
         dispatchEvent(new InteractiveScene3DEvent(InteractiveScene3DEvent.OBJECT_PRESS,containerDictionary[param1.currentTarget],Sprite(param1.currentTarget)));
      }
      
      protected function handleMouseOut(param1:MouseEvent) : void
      {
         if(debug)
         {
            log.debug("Out",DisplayObject3D(containerDictionary[param1.target]).name);
         }
         dispatchEvent(new InteractiveScene3DEvent(InteractiveScene3DEvent.OBJECT_OUT,containerDictionary[param1.currentTarget],Sprite(param1.currentTarget)));
      }
      
      protected function handleResize(param1:Event) : void
      {
         resizeStage();
      }
      
      protected function handleReleaseOutside(param1:MouseEvent) : void
      {
         if(debug)
         {
            log.debug("releaseOutside");
         }
         dispatchEvent(new InteractiveScene3DEvent(InteractiveScene3DEvent.OBJECT_RELEASE_OUTSIDE));
      }
   }
}

